<?php

class Zippy_Product_Utilities
{
    public function getCoursesAndTiers($product_id)
    {
        $zippy = Zippy::instance();

        if ($product_id === null) {
            return array();
        }

        $courses_and_tiers = array();

        $access = $zippy->utilities->getJsonMeta($product_id, 'access');
        if (!is_object($access) || !isset($access->mode)) {
            return $courses_and_tiers;
        }

        if ($access->mode == 'product') {
            $courses_and_tiers[] = array(
                'id' => (int) $access->course,
                'tiers' => array((int) $access->tier)
            );
        } else {
            if (isset($access->bundle) && is_array($access->bundle)) {
                foreach ($access->bundle as $course) {
                    $courses_and_tiers[] = array(
                        'id' => (int) $course->id,
                        'tiers' => $course->tiers
                    );
                }
            }
        }

        return $courses_and_tiers;
    }

    public function getCourses($product_id)
    {
        $courses_and_tiers = $this->getCoursesAndTiers($product_id);

        $courses = array();

        foreach ($courses_and_tiers as $course) {
            if (!empty($course['tiers'])) {
                $courses[] = $course['id'];
            }
        }

        return $courses;
    }

    public function getTiers($product_id)
    {
        $courses_and_tiers = $this->getCoursesAndTiers($product_id);

        $tiers = array();

        foreach ($courses_and_tiers as $course) {
            $tiers[$course['id']] = $course['tiers'];
        }

        return $tiers;
    }

    public function getAll()
    {
        global $wpdb;

        $zippy = Zippy::instance();

        $invalid_post_statuses = $zippy->utilities->getInvalidPostStatus();
        $invalid_post_status_list = '';

        foreach ($invalid_post_statuses as $post_status) {
            $invalid_post_status_list .= '"' . $post_status . '",';
        }

        $invalid_post_status_list = rtrim($invalid_post_status_list, ',');

        $sql = $wpdb->prepare("SELECT ID FROM $wpdb->posts WHERE post_type = %s AND post_status NOT IN ($invalid_post_status_list)", 'product');

        $products       = $zippy->make('products');
        $product_ids    = $wpdb->get_col($sql);

        foreach ($product_ids as $product_id) {
            $product = $zippy->make('product', array($product_id));
            $products->add($product);
        }

        return $products;
    }

    public function getBySlug($slug)
    {
        global $wpdb;

        $zippy = Zippy::instance();
        $sql = $wpdb->prepare("SELECT ID FROM $wpdb->posts WHERE post_name = %s AND post_type = 'product' LIMIT 1", $slug);

        $id = $wpdb->get_var($sql);

        return $this->getById($id);
    }

    public function getByUid($uid)
    {
        global $wpdb;

        $zippy = Zippy::instance();
        $sql = $wpdb->prepare(
            "SELECT
                post_id
             FROM
                $wpdb->postmeta
             WHERE
                meta_key = %s AND
                meta_value = %s
            LIMIT 1",
            'uid',
            $uid
        );
    
        $id = $wpdb->get_var($sql);

        return $id ? $this->getById($id) : null;
    }

    public function getById($id)
    {
        global $wpdb;

        if (!$id) {
            return null;
        }

        $zippy = Zippy::instance();

        $product = get_post($id);

        return $product !== null && $product->post_type == 'product' ? $zippy->make('product', array($id)) : null;
    }

    public function inLaunchWindow(Zippy_Product $product, $date = null)
    {
        if (!$product->getLaunchWindowsEnabled()) {
            return true;
        }
    
        $zippy          = Zippy::instance();

        $date           = $date instanceof DateTime ? $date : $zippy->utilities->datetime->getNow();
        $launch_windows = $product->getLaunchWindows();

        $in_window = false;

        foreach ($launch_windows as $launch_window) {
            $open  = $zippy->utilities->datetime->getDate($launch_window->open_date);
            $close = $zippy->utilities->datetime->getDate($launch_window->close_date);

            $in_window = $date->format('U') >= $open->format('U') && $date->format('U') <= $close->format('U');

            if ($in_window) {
                break;
            }
        }

        return $in_window;
    }

    public function getProductsNotOwned(ZippyCourses_Student $student)
    {
        global $wpdb;

        $zippy = Zippy::instance();

        $invalid_post_statuses = $zippy->utilities->getInvalidPostStatus();
        $invalid_post_status_list = '';

        $products_owned = $student->getProductsOwned();

        foreach ($invalid_post_statuses as $post_status) {
            $invalid_post_status_list .= '"' . $post_status . '",';
        }

        $invalid_post_status_list = rtrim($invalid_post_status_list, ',');

        $sql = $wpdb->prepare("SELECT ID FROM $wpdb->posts WHERE post_type = %s AND post_status NOT IN ($invalid_post_status_list)", 'product');
        $product_ids = $wpdb->get_col($sql);

        $products = $zippy->make('products');
        foreach ($product_ids as $product_id) {
            if (!in_array($product_id, $products_owned)) {
                $product = $zippy->make('product', array($product_id));
                $products->add($product);
            }
        }

        return $products;
    }

    public function getProductByBuyQueryVar()
    {
        global $post;

        $buy = trim(get_query_var('buy')) != '' ? get_query_var('buy') : null;

        $product = $this->getById($buy);

        if ($product === null && $buy) {
            $product = $this->getByUid($buy);
        }

        if ($product === null && $buy) {
            $product = $this->getBySlug($buy);
        }

        return $product;
    }

    public function getAllProductIds()
    {
        global $wpdb;
            
        $zippy = Zippy::instance();

        $query_id = 'zippy_get_all_product_ids';

        $invalid_post_statuses = array('"auto-draft"','"trash"', '"inherit"');
        $invalid_post_status_list = implode(', ', $invalid_post_statuses);

        if (($query = $zippy->queries->get($query_id)) === null) {
            global $wpdb;

            $query = $zippy->make('query', array('id' => $query_id));
            $query->setSql("SELECT ID FROM $wpdb->posts WHERE post_type = 'product' AND post_status NOT IN ($invalid_post_status_list)");

            $zippy->queries->add($query);
        }

        return $query->getCol();
    }

    public function getCount()
    {
        global $wpdb;

        $sql = "SELECT COUNT(*) FROM $wpdb->posts WHERE post_type = 'product'";

        return $wpdb->get_var($sql);
    }
}
