<?php
/**
 * Zippy Courses functions and definitions
 *
 * Set up the theme and provides some helper functions, which are used in the
 * theme as custom template tags. Others are attached to action and filter
 * hooks in WordPress to change core functionality.
 */

class ZippyCoursesTheme
{
    const TEXTDOMAIN = 'zippy-courses-theme';
}

require_once(dirname(__FILE__) . '/updater.php');

if( !class_exists( 'Zippy_Theme_Layout' ) ) {
    // load our custom theme layout tool
    include( dirname( __FILE__ ) . '/lib/layout.php' );
}
 
if( !defined( 'ZC_THEME_DIR' ) ) {
    define( 'ZC_THEME_DIR', dirname( __FILE__ ) );
}

if( !defined( 'ZC_THEME_NAMESPACE' ) ) {
    define( 'ZC_THEME_NAMESPACE', 'zippy-courses-theme');
}

if( !defined( 'ZC_THEME_URL' ) ) {
    define( 'ZC_THEME_URL', get_template_directory_uri()  );
}

function zippy_lang_setup(){
    load_theme_textdomain('zippy-courses-theme', get_template_directory() . '/lang');
}
add_action('after_setup_theme', 'zippy_lang_setup');

if ( ! function_exists( 'zippy_theme_setup' ) ) :

/**
 * Zippy Courses setup.
 *
 * Set up theme defaults and registers support for various WordPress features.
 *
 * Note that this function is hooked into the after_setup_theme hook, which
 * runs before the init hook. The init hook is too late for some features, such
 * as indicating support post thumbnails.
 *
 * @since WP Zippy Courses 1.0
 */
function zippy_theme_setup() {

    require_once( dirname( __FILE__ ) . '/lib/deprecated.php' );

    // Add RSS feed links to <head> for posts and comments.
    add_theme_support( 'automatic-feed-links' );

    // Enable BBPress Support
    add_theme_support( 'bbpress' );

    add_theme_support( 'title-tag' );
    
    // Enable support for Post Thumbnails, and declare two sizes.
    add_theme_support( 'post-thumbnails' );

    // This theme uses wp_nav_menu() in two locations.
    register_nav_menus( array(
        'primary'   => __( 'Primary (Top) Menu ', ZippyCoursesTheme::TEXTDOMAIN)
    ) );

    /*
     * Switch default core markup for search form, comment form, and comments
     * to output valid HTML5.
     */
    add_theme_support( 'html5', array( 'search-form', 'comment-form', 'comment-list' ) );

    // This theme uses its own gallery styles.
    add_filter( 'use_default_gallery_style', '__return_false' );

}
endif; // zippy_courses_theme_setup
add_action( 'after_setup_theme', 'zippy_theme_setup' );

if( is_admin() ) {
    require_once( dirname( __FILE__ ) . '/lib/admin.php' );
}

/**
 * Register three Zippy Courses widget areas.
 *
 * @since Zippy Courses 0.9
 *
 * @return void
 */
function zippy_courses_theme_widgets_init() {

    register_sidebar( array(
        'name'          => __( 'Primary Sidebar', ZippyCoursesTheme::TEXTDOMAIN),
        'id'            => 'sidebar-1',
        'description'   => __( 'Main sidebar that appears on the left.','zippy-courses-theme'),
        'before_widget' => '<aside id="%1$s" class="widget %2$s">',
        'after_widget'  => '</aside>',
        'before_title'  => '<h3 class="widget-title">',
        'after_title'   => '</h3>',
    ) );

    register_sidebar( array(
        'name'          => __( 'Footer Widget Area', ZippyCoursesTheme::TEXTDOMAIN),
        'id'            => 'sidebar-2',
        'description'   => __( 'Appears in the footer section of the site.', ZippyCoursesTheme::TEXTDOMAIN),
        'before_widget' => '<aside id="%1$s" class="widget %2$s">',
        'after_widget'  => '</aside>',
        'before_title'  => '<h3 class="widget-title">',
        'after_title'   => '</h3>',
    ) );

}
add_action( 'widgets_init', 'zippy_courses_theme_widgets_init' );

/**
 * Enqueue scripts and styles for the front end.
 *
 * @since Zippy Courses 0.9
 *
 * @return void
 */
function zippy_courses_theme_scripts() {

    global $current_user, $zippy_courses_theme_event_id;

    wp_enqueue_script('jquery');
    wp_enqueue_script( 'bootstrap-js', '//netdna.bootstrapcdn.com/bootstrap/3.0.3/js/bootstrap.min.js', array( 'jquery' ), ZC_THEME_VERSION, true );
    wp_enqueue_script( 'wpcs-plugins-js', ZC_THEME_URL . '/assets/js/plugins.js', array( 'bootstrap-js', 'jquery' ), ZC_THEME_VERSION, true );
    wp_enqueue_script( 'wpcs-js', ZC_THEME_URL . '/assets/js/wpcs.js', array( 'wpcs-plugins-js' ), ZC_THEME_VERSION, true );

    wp_localize_script( 'wpcs-js', 'CourseSoftware' , array( 
        'ajaxurl' => admin_url( 'admin-ajax.php' ),
        'student_id' => $current_user->ID,
        'event_id' => $zippy_courses_theme_event_id
    ));

    // Load our main stylesheet.
    wp_enqueue_style( 'zippy-courses-style', get_stylesheet_uri() );

}
add_action( 'wp_enqueue_scripts', 'zippy_courses_theme_scripts' );

/**
 * Extend the default WordPress body classes.
 *
 * @since Zippy Courses 0.9
 *
 * @param array $classes A list of existing body class values.
 * @return array The filtered body class list.
 */
function zippy_courses_theme_body_classes( $classes ) {

    global $post, $template;

    if( basename( $template ) == 'page-login.php' )
        $classes[] = 'zippy-login-page';

    return $classes;
}
add_filter( 'body_class', 'zippy_courses_theme_body_classes' );

/**
 * This removes the attribution from the wp_footer 
 * because this theme uses it in zippy_footer_bottom
 * @return void
 */
function zippy_theme_remove_attribution_from_footer() {

    global $wp_filter;

    foreach( $wp_filter as $hook => $priorities ) {

        if( $hook == 'wp_footer' ) {

            foreach( $priorities as $priority => $functions ) {

                foreach( $functions as $name => $function ) {

                    if( is_array( $function['function'] ) ) {

                        if( strpos( $name, 'zippy_attribution' ) !== false )
                            unset( $wp_filter[ $hook ][ $priority ][ $name ] );

                    }
                }
            }
        }
    }
}
add_action( 'template_redirect', 'zippy_theme_remove_attribution_from_footer' );

function zippy_theme_favicon()
{
    $favicon = get_theme_mod('options_favicon_file', false);
    if ($favicon) {
        echo '<link rel="icon" type="image/ico" href="' . $favicon . '"/>';
    }        
}
add_action('wp_head', 'zippy_theme_favicon');

function zippy_courses_theme_fonts() {
    echo '<link href="//fonts.googleapis.com/css?family=Open+Sans:600,400,300" rel="stylesheet" type="text/css">';
}
add_action( 'wp_head', 'zippy_courses_theme_fonts' );

function zippy_courses_theme_course_header( $post_id = 0 ) {
    global $post;
    $post_id = $post_id ? $post_id : $post->ID;
    echo zippy_courses_theme_get_course_header( $post_id );
}

function zippy_theme_show_hide_lessons_toggle($html, $unit)
{
    $output = '<a class="zippy-theme-unit-lesson-toggle">[' . __('Show Lessons', ZippyCoursesTheme::TEXTDOMAIN) . ']</a>';

    return $output . $html;
}
add_filter('zippy_filter_course_entries_unit_title_html', 'zippy_theme_show_hide_lessons_toggle', 10, 2);

function next_posts_link_attributes() {
    return 'class="next-posts"';
}
add_filter('next_posts_link_attributes', 'next_posts_link_attributes');

function prev_posts_link_attributes() {
    return 'class="prev-posts"';
}
add_filter('previous_posts_link_attributes', 'prev_posts_link_attributes');

function get_zippy_courses_theme_logo( $type = 'default' ) {

    // Changed the Theme customizer details
    $old = get_theme_mod( 'zippy_courses_theme_logo', '' );
    $new = $type == 'login' 
            ? ( get_theme_mod( 'logo_login', '' ) == '' 
                    ? get_theme_mod( 'logo_primary', '' )
                    : get_theme_mod( 'logo_login', '' ) )
            : get_theme_mod( 'logo_primary', '' );
    $default = '';

    $logo = $old == '' 
        ? ( $new == '' ? $default : $new ) 
        : ( $new == '' ? $old : $new );

    $html = '';

    if( $logo == '' ) { 
        $html = get_bloginfo( 'name' );
    } else {
        
        global $wpdb;

        $img = $wpdb->get_row( $wpdb->prepare( "SELECT * FROM $wpdb->posts WHERE guid = %s", $logo ) );
        $src = wp_get_attachment_image_src( $img->ID, 'full' );
        $html = '<img src="' . $src[0] . '" class="logo" width="' . $src[1] . '" height="' . $src[2] . '">';

    }

    return $html;

}

function zippy_courses_theme_logo() {  
    echo get_zippy_courses_theme_logo();
}

function zippy_customizer_render_css() {
    require_once dirname( __FILE__ ) . '/lib/customizer-css.php';
}
add_action( 'wp_head', 'zippy_customizer_render_css' );

function zippy_theme_activation() {
  
    $check = get_option('zippy_theme_activation_check');

    if ( $check != "set" ) {
        // DO INITIALISATION STUFF

        // Add marker so it doesn't run in future
        add_option('zippy_theme_activation_check', "set");

    }

}
add_action('init', 'zippy_theme_activation');

function zippy_theme_deactivation($newname, $newtheme) {
    delete_option( 'zippy_theme_activation_check' );
}
add_action("switch_theme", "zippy_theme_deactivation", 10 , 2);  

function zippy_courses_theme_tint_color( $color, $amount ) {

    $color_r = substr( str_replace( '#', '', $color ), 0, 2 );
    $color_r_hover = dechex ( floor( hexdec( $color_r ) * $amount ) );

    $color_g = substr( str_replace( '#', '', $color ), 2, 2 );
    $color_g_hover = dechex ( floor( hexdec( $color_g ) * $amount ) );

    $color_b = substr( str_replace( '#', '', $color ), 4, 2 );
    $color_b_hover = dechex ( floor( hexdec( $color_b ) * $amount ) );

    return '#' . $color_r_hover . $color_g_hover . $color_b_hover;

}

function zippy_theme_reset_password( $output, $user_id ) {

    $output = '<form action="" class="form-horizontal" method="POST" role="form">' . "\n" . 
          '<div class="form-group">' . "\n" . 
            '<label for="user_pass1"  class="col-sm-4 control-label">' . __('Password', ZippyCoursesTheme::TEXTDOMAIN) . '</label>' . "\n" .
            '<div class="col-sm-6">' . "\n" .
            '<input type="password" class="form-control" id="user_pass1" name="user_pass1" placeholder="' . __('Password', ZippyCoursesTheme::TEXTDOMAIN) . '">' . "\n" . 
            '</div>' . "\n" .
          '</div>' . "\n" . 
          '<div class="form-group">' . "\n" . 
            '<label for="user_pass2"  class="col-sm-4 control-label">' . __('Confirm Password', ZippyCoursesTheme::TEXTDOMAIN) . '</label>' . "\n" . 
            '<div class="col-sm-6">' . "\n" .
            '<input type="password" class="form-control" id="user_pass2" name="user_pass2" placeholder="' . __('Confirm Password', ZippyCoursesTheme::TEXTDOMAIN) . '">' . "\n" . 
            '</div>' . "\n" .
          '</div>' . "\n" . 
          '<div class="col-sm-offset-4 col-sm-6">' . "\n" .
            '<input type="hidden" name="user" value="' . $user_id . '" />' . "\n" . 
            '<input type="hidden" name="course_action" value="reset_password" />' . "\n" . 
            '<p><button type="submit" class="btn btn-primary">' . __('Reset Password', ZippyCoursesTheme::TEXTDOMAIN) . '</button></p>' . "\n" . 
          '</div>' . "\n" . 
        '</form>' . "\n";

    return $output;

}

add_action( 'bbp_template_before_single_topic', 'zippy_bbp_topic_title' );
function zippy_bbp_topic_title() {
    
    global $post;

    $post_type_object = get_post_type_object($post->post_type);

    $html = '<header class="entry-header row">';

        $html .= '<div class="col-sm-12"><h2 class="entry-title"><span class="post-type-indicator">' . apply_filters( 'zippy_theme_post_type_indicator', $post_type_object->labels->singular_name) . '</span>' . $post->post_title . '</h2>' . bbp_get_topic_subscription_link( array( 'before' => '' )) . ' ' . bbp_get_user_favorites_link() . '</div>';

    $html .= '</header>';

    echo $html;

}

function zippy_bbp_search_title() {
    
    global $post;

    $html = '<header class="entry-header row">';

        $html .= '<div class="col-sm-12"><h2 class="entry-title"><span class="post-type-indicator">' . apply_filters( 'zippy_theme_post_type_indicator', __('Search Results for:', ZippyCoursesTheme::TEXTDOMAIN) ) . '</span>' . bbp_get_search_terms() . '</h2></div>';

    $html .= '</header>';

    echo $html;

}
add_action( 'bbp_template_before_search', 'zippy_bbp_search_title' );

function zippy_bbp_topics_tag_title() {
    
    global $post;

    $html = '<header class="entry-header row">';

        $html .= '<div class="col-sm-12"><h2 class="entry-title"><span class="post-type-indicator">' . apply_filters( 'zippy_theme_post_type_indicator', __('Tagged:', ZippyCoursesTheme::TEXTDOMAIN) ) . '</span>' . bbp_get_topic_tag_name() . '</h2></div>';

    $html .= '</header>';

    echo $html;

}
add_action( 'bbp_template_before_topics_index', 'zippy_bbp_topics_tag_title' );

function zippy_bbp_forum_title() {
    
    global $post;
    $post_type_object = get_post_type_object($post->post_type);

    $html = '<header class="entry-header row">';

        $html .= '<div class="col-sm-12"><h2 class="entry-title"><span class="post-type-indicator">' . apply_filters( 'zippy_theme_post_type_indicator', $post_type_object->labels->singular_name ) . '</span>' . $post->post_title . '</h2>' . bbp_get_forum_subscription_link( array( 'before' => '' )) . ' ' . bbp_get_user_favorites_link() . '</div>';


    $html .= '</header>';

    echo $html;

}
add_action( 'bbp_template_before_single_forum', 'zippy_bbp_forum_title' );

function zippy_bbp_forums_title() {
    
    global $post;

    $html = '<header class="entry-header row">';

        $html .= '<div class="col-sm-12"><h2 class="entry-title"><span class="post-type-indicator">' . apply_filters( 'zippy_theme_post_type_indicator', __('Forums', ZippyCoursesTheme::TEXTDOMAIN) ) . '</span></h2></div>';

    $html .= '</header>';

    echo $html;

}
add_action( 'bbp_template_before_forums_index', 'zippy_bbp_forums_title' );

function zippy_bbp_replace_title( $title ) {
    
    if( is_bbpress() )
        $title = __('Forums', ZippyCoursesTheme::TEXTDOMAIN);

    return $title;

}
// add_filter( 'the_title', 'zippy_bbp_replace_title' );

add_action( 'bbp_get_single_topic_description', create_function( '', "return '';" ));
add_action( 'bbp_get_single_forum_description', create_function( '', "return '';" ));

function zippy_bbp_breadcrumbs_and_search() {
    ?>
    
    <div class="zippy-bbp-top-nav">
        <div class="container">
            <div class="row">
                <div class="bbp-breadcrumb col-md-9">
                    <?php bbp_breadcrumb( array( 'before' => '', 'after'  => '' )); ?>
                </div>
                <?php if ( bbp_allow_search() ) : ?>
                <div class="bbp-search-form col-md-3">
                    <?php bbp_get_template_part( 'form', 'search' ); ?>
                </div>
                <?php endif; ?>             
            </div>
        </div>
    </div>
    
    <?php
}

/**
 * Class to create a custom menu control
 */


function zippy_theme_customizer_setup( $wp_customize ) {

    require_once( dirname( __FILE__ ) . '/lib/customizer.php' );

    $wp_customize->remove_section( 'background_image' );
    $wp_customize->remove_section( 'colors' );

    $defaults = array(
        'black'         => '#000000',
        'white'         => '#FFFFFF',
        'eggshell'      => '#FAFAFA',
        'teal'          => '#15D3EB',
        'dark_gray'     => '#222222',
        'light_gray'    => '#CCCCCC',
        'lighter_gray'  => '#DDDDDD',
        'lightest_gray' => '#EEEEEE',
        'gray'          => '#777777',
        'dark_blue'     => '#222233',
        'blue'          => '#3b7db5',
        'green'         => '#33AA55'
    );

    $zc = new Zippy_Theme_Customizer(); 

    $options = $zc->add_panel( 'options', 'Theme Options', '', 110 );
        $favicon_options = $options->add_section( 'favicon', __('Favicon', ZippyCoursesTheme::TEXTDOMAIN) );
        $favicon_options->add_control( 'file', __('File', ZippyCoursesTheme::TEXTDOMAIN), __('This file must be a .ico to function properly', ZippyCoursesTheme::TEXTDOMAIN) )->set_type('upload');

    $layout = $zc->add_panel( 'layout', __('Default Layouts', ZippyCoursesTheme::TEXTDOMAIN), '', 110 );

        $posts_and_pages_layout = $layout->add_section( 'posts_and_pages', __('Posts and Pages', ZippyCoursesTheme::TEXTDOMAIN) );
        $posts_and_pages_layout->add_setting_filter( 'name', array( 'zippy_basic_theme_layout', 'zippy' ) );

            $posts_and_pages_layout->add_control( 'layout', __('Layout', ZippyCoursesTheme::TEXTDOMAIN), __('Change the default layout for Posts and Pages', ZippyCoursesTheme::TEXTDOMAIN) )->set_choices( array( __('Right Sidebar', ZippyCoursesTheme::TEXTDOMAIN), __('Left Sidebar', ZippyCoursesTheme::TEXTDOMAIN), __('No Sidebar', ZippyCoursesTheme::TEXTDOMAIN) ) )->set_type( 'select' );
            $posts_and_pages_layout->add_control( 'menu_visibility', __('Menu', ZippyCoursesTheme::TEXTDOMAIN), __('Is the menu visible?', ZippyCoursesTheme::TEXTDOMAIN) )->set_choices( array( '1' => __('On', ZippyCoursesTheme::TEXTDOMAIN), '0' => __('Off', ZippyCoursesTheme::TEXTDOMAIN) ) )->set_type( 'select' );
            $posts_and_pages_layout->add_control( 'header_visibility', __('Header', ZippyCoursesTheme::TEXTDOMAIN), __('Is the header visible?', ZippyCoursesTheme::TEXTDOMAIN) )->set_choices( array( '1' => __('On', ZippyCoursesTheme::TEXTDOMAIN), '0' => __('Off', ZippyCoursesTheme::TEXTDOMAIN) ) )->set_type( 'select' );
            $posts_and_pages_layout->add_control( 'footer_visibility', __('Footer', ZippyCoursesTheme::TEXTDOMAIN), __('Is the footer visible?', ZippyCoursesTheme::TEXTDOMAIN) )->set_choices( array( '1' => __('On', ZippyCoursesTheme::TEXTDOMAIN), '0' => __('Off', ZippyCoursesTheme::TEXTDOMAIN) ) )->set_type( 'select' );

        $courses_layout         = $layout->add_section( 'courses', __('Courses', ZippyCoursesTheme::TEXTDOMAIN) );
        $courses_layout->add_setting_filter( 'name', array( 'zippy_basic_theme_layout', 'zippy' ) );

            $courses_layout->add_control( 'layout', __('Layout', ZippyCoursesTheme::TEXTDOMAIN), __('Change the default layout for Posts and Pages', ZippyCoursesTheme::TEXTDOMAIN) )->set_choices( array( __('Right Sidebar', ZippyCoursesTheme::TEXTDOMAIN), __('Left Sidebar', ZippyCoursesTheme::TEXTDOMAIN), __('No Sidebar', ZippyCoursesTheme::TEXTDOMAIN) ) )->set_type( 'select' );
            $courses_layout->add_control('menu_visibility', __('Menu', ZippyCoursesTheme::TEXTDOMAIN), __('Is the menu visible?', ZippyCoursesTheme::TEXTDOMAIN) )->set_choices(array('1' => __('On', ZippyCoursesTheme::TEXTDOMAIN), '0' => __('Off', ZippyCoursesTheme::TEXTDOMAIN)))->set_type('select');
            $courses_layout->add_control('header_visibility', __('Header', ZippyCoursesTheme::TEXTDOMAIN), __('Is the header visible?', ZippyCoursesTheme::TEXTDOMAIN) )->set_choices( array('1' => __('On', ZippyCoursesTheme::TEXTDOMAIN), '0' => __('Off', ZippyCoursesTheme::TEXTDOMAIN)))->set_type( 'select');
            $courses_layout->add_control('footer_visibility', __('Footer', ZippyCoursesTheme::TEXTDOMAIN), __('Is the footer visible?', ZippyCoursesTheme::TEXTDOMAIN) )->set_choices( array( '1' => __('On', ZippyCoursesTheme::TEXTDOMAIN), '0' => __('Off', ZippyCoursesTheme::TEXTDOMAIN) ) )->set_type( 'select');

        $units_layout           = $layout->add_section( 'units', __('Units', ZippyCoursesTheme::TEXTDOMAIN) );
        $units_layout->add_setting_filter( 'name', array( 'zippy_basic_theme_layout', 'zippy' ) );

            $units_layout->add_control( 'layout', __('Layout', ZippyCoursesTheme::TEXTDOMAIN), __('Change the default layout for Posts and Pages', ZippyCoursesTheme::TEXTDOMAIN) )->set_choices( array( __('Right Sidebar', ZippyCoursesTheme::TEXTDOMAIN), __('Left Sidebar', ZippyCoursesTheme::TEXTDOMAIN), __('No Sidebar', ZippyCoursesTheme::TEXTDOMAIN) ) )->set_type( 'select' );
            $units_layout->add_control( 'menu_visibility', 'Menu', 'Is the menu visible?' )->set_choices( array( '1' => __('On', ZippyCoursesTheme::TEXTDOMAIN), '0' => __('Off', ZippyCoursesTheme::TEXTDOMAIN) ) )->set_type( 'select' );
            $units_layout->add_control( 'header_visibility', 'Header', 'Is the header visible?' )->set_choices( array( '1' => __('On', ZippyCoursesTheme::TEXTDOMAIN), '0' => __('Off', ZippyCoursesTheme::TEXTDOMAIN) ) )->set_type( 'select' );
            $units_layout->add_control( 'footer_visibility', 'Footer', 'Is the footer visible?' )->set_choices( array( '1' => __('On', ZippyCoursesTheme::TEXTDOMAIN), '0' => __('Off', ZippyCoursesTheme::TEXTDOMAIN) ) )->set_type( 'select' );

        $lessons_layout         = $layout->add_section( 'lessons', 'Lessons' );
        $lessons_layout->add_setting_filter( 'name', array( 'zippy_basic_theme_layout', 'zippy' ) );

            $lessons_layout->add_control( 'layout', 'Layout', 'Change the default layout for Posts and Pages' )->set_choices( array( __('Right Sidebar', ZippyCoursesTheme::TEXTDOMAIN), __('Left Sidebar', ZippyCoursesTheme::TEXTDOMAIN), __('No Sidebar', ZippyCoursesTheme::TEXTDOMAIN) ) )->set_type( 'select' );
            $lessons_layout->add_control( 'menu_visibility', 'Menu', 'Is the menu visible?' )->set_choices( array( '1' => __('On', ZippyCoursesTheme::TEXTDOMAIN), '0' => __('Off', ZippyCoursesTheme::TEXTDOMAIN) ) )->set_type( 'select' );
            $lessons_layout->add_control( 'header_visibility', 'Header', 'Is the header visible?' )->set_choices( array( '1' => __('On', ZippyCoursesTheme::TEXTDOMAIN), '0' => __('Off', ZippyCoursesTheme::TEXTDOMAIN) ) )->set_type( 'select' );
            $lessons_layout->add_control( 'footer_visibility', 'Footer', 'Is the footer visible?' )->set_choices( array( '1' => __('On', ZippyCoursesTheme::TEXTDOMAIN), '0' => __('Off', ZippyCoursesTheme::TEXTDOMAIN) ) )->set_type( 'select' );

    $colors = $zc->add_panel( 'colors', __('Colors', ZippyCoursesTheme::TEXTDOMAIN), '', 105 );

        $site_colors = $colors->add_section( 'site', __('Site', ZippyCoursesTheme::TEXTDOMAIN) );
            $site_colors->add_control( 'bg', __('Background Color', ZippyCoursesTheme::TEXTDOMAIN) )->set_type( 'color' )->set_default( $defaults['eggshell'] );
            $site_colors->add_control( 'text', __('Text Color', ZippyCoursesTheme::TEXTDOMAIN) )->set_type( 'color' )->set_default( $defaults['dark_gray'] );
            $site_colors->add_control( 'link', __('Link Color', ZippyCoursesTheme::TEXTDOMAIN) )->set_type( 'color' )->set_default( $defaults['blue'] );
            $site_colors->add_control( 'button_primary_bg', __('Primary Button Background', ZippyCoursesTheme::TEXTDOMAIN) )->set_type( 'color' )->set_default( $defaults['blue'] );
            $site_colors->add_control( 'button_primary_text', __('Primary Button Text Color', ZippyCoursesTheme::TEXTDOMAIN) )->set_type( 'color' )->set_default( $defaults['white'] );
            $site_colors->add_control( 'button_secondary_bg', __('Secondary Button Background', ZippyCoursesTheme::TEXTDOMAIN) )->set_type( 'color' )->set_default( $defaults['lighter_gray'] );
            $site_colors->add_control( 'button_secondary_text', __('Secondary Button Text Color', ZippyCoursesTheme::TEXTDOMAIN) )->set_type( 'color' )->set_default( $defaults['gray'] );
            $site_colors->add_control( 'button_complete_bg', __('Complete Lesson Button Background', ZippyCoursesTheme::TEXTDOMAIN) )->set_type( 'color' )->set_default( $defaults['green'] );
            $site_colors->add_control( 'button_complete_text', __('Complete Lesson Button Text Color', ZippyCoursesTheme::TEXTDOMAIN) )->set_type( 'color' )->set_default( $defaults['white'] );
            
        $login_colors = $colors->add_section( 'login', __('Login', ZippyCoursesTheme::TEXTDOMAIN), 'These colors will be used on your Login Page template and no others.' );
            $login_colors->add_control( 'bg' , __('Background Color', ZippyCoursesTheme::TEXTDOMAIN) )->set_type( 'color' )->set_default( $defaults['eggshell'] );
            $login_colors->add_control( 'form_bg' , __('Form Background Color', ZippyCoursesTheme::TEXTDOMAIN) )->set_type( 'color' )->set_default( $defaults['white'] );
            $login_colors->add_control( 'text', __('Text Color', ZippyCoursesTheme::TEXTDOMAIN) )->set_type( 'color' )->set_default( $defaults['dark_gray'] );
            $login_colors->add_control( 'link', __('Login Link Color', ZippyCoursesTheme::TEXTDOMAIN) )->set_type( 'color' )->set_default( $defaults['blue'] );
            $login_colors->add_control( 'button_bg', __('Login Button Background Color', ZippyCoursesTheme::TEXTDOMAIN) )->set_type( 'color' )->set_default( $defaults['blue'] );
            $login_colors->add_control( 'button_text', __('Login Button Text Color', ZippyCoursesTheme::TEXTDOMAIN) )->set_type( 'color' )->set_default( $defaults['white'] );

        $menu_colors = $colors->add_section( 'menu', __('Menu', ZippyCoursesTheme::TEXTDOMAIN) );
            $menu_colors->add_control( 'bg', __('Background Color', ZippyCoursesTheme::TEXTDOMAIN) )->set_type( 'color' )->set_default( $defaults['dark_gray'] );
            $menu_colors->add_control( 'link', __('Link Color', ZippyCoursesTheme::TEXTDOMAIN) )->set_type( 'color' )->set_default( $defaults['light_gray'] );

        $header_colors = $colors->add_section( 'header', __('Primary Header', ZippyCoursesTheme::TEXTDOMAIN), 'The <strong>Primary Header</strong> appears directly below the top menu, and is separate from the <strong>Content Header</strong>.' );
            $header_colors->add_control( 'bg', __('Background Color', ZippyCoursesTheme::TEXTDOMAIN) )->set_type( 'color' )->set_default( $defaults['teal'] );
            $header_colors->add_control( 'text', __('Text Color', ZippyCoursesTheme::TEXTDOMAIN) )->set_type( 'color' )->set_default( $defaults['white'] );

        $content_colors = $colors->add_section( 'content', __('Content', ZippyCoursesTheme::TEXTDOMAIN), 'These colors are used to change the colors for the headings and content of posts, course, unit or lesson. To change the Page Header, use the <strong>Primary Header</strong> section in this panel and to change Text or Link color, use the <strong>Site</strong> section.' );
            $content_colors->add_control( 'bg', __('Background Color', ZippyCoursesTheme::TEXTDOMAIN) )->set_type( 'color' )->set_default( $defaults['white'] );
            $content_colors->add_control( 'header_bg', __('Header Background Color', ZippyCoursesTheme::TEXTDOMAIN) )->set_type( 'color' )->set_default( $defaults['dark_blue'] );
            $content_colors->add_control( 'header_text', __('Header Text Color', ZippyCoursesTheme::TEXTDOMAIN) )->set_type( 'color' )->set_default( $defaults['white'] );
            $content_colors->add_control( 'footer_bg', __('Footer Background Color', ZippyCoursesTheme::TEXTDOMAIN) )->set_type( 'color' )->set_default( $defaults['lighter_gray'] );
            $content_colors->add_control( 'footer_text', __('Footer Text Color', ZippyCoursesTheme::TEXTDOMAIN) )->set_type( 'color' )->set_default( $defaults['dark_gray'] );

        $sidebar_colors = $colors->add_section( 'sidebar', __('Sidebar', ZippyCoursesTheme::TEXTDOMAIN) );
            $sidebar_colors->add_control( 'heading', __('Heading Color', ZippyCoursesTheme::TEXTDOMAIN) )->set_type( 'color' )->set_default( $defaults['dark_gray'] );

        $footer_colors = $colors->add_section( 'footer', __('Footer', ZippyCoursesTheme::TEXTDOMAIN) );
            $footer_colors->add_control( 'bg', __('Background Color', ZippyCoursesTheme::TEXTDOMAIN) )->set_type( 'color' )->set_default( $defaults['black'] );
            $footer_colors->add_control( 'text', __('Text Color', ZippyCoursesTheme::TEXTDOMAIN) )->set_type( 'color' )->set_default( $defaults['lighter_gray'] );
            $footer_colors->add_control( 'link', __('Link Color', ZippyCoursesTheme::TEXTDOMAIN) )->set_type( 'color' )->set_default( $defaults['lightest_gray'] );

    $logos = $zc->add_section( 'logo', __('Logos', ZippyCoursesTheme::TEXTDOMAIN), '', 30 );
        $logos->add_control( 'primary', __('Primary Logo', ZippyCoursesTheme::TEXTDOMAIN), __('This logo should be no taller than 40px.  It will appear on the left in the menu bar. If you do not set a Login Page Logo below, it will also display on the Login Page template.', ZippyCoursesTheme::TEXTDOMAIN) )->set_type( 'image' );
        $logos->add_control( 'login', __('Login Page Logo', ZippyCoursesTheme::TEXTDOMAIN), __('This logo can be larger, up to 300px wide and as tall as you would like.', ZippyCoursesTheme::TEXTDOMAIN) )->set_type( 'image' );
    /*
    $zippycourses = $zc->add_panel( 'zippy_courses', __('Zippy Courses Options', ZippyCoursesTheme::TEXTDOMAIN), '', 110 );

        $dashboard  = $zippycourses->add_section('course_dashboard', __('Course Dashboard', ZippyCoursesTheme::TEXTDOMAIN));
        $dashboard->add_control( 'show_excerpt', __('Show Unit & Lesson Excerpt?', ZippyCoursesTheme::TEXTDOMAIN), '' )->set_choices( array( '1' => __('Yes', ZippyCoursesTheme::TEXTDOMAIN), '0' => __('No', ZippyCoursesTheme::TEXTDOMAIN) ) )->set_type( 'select' );
        $dashboard->add_control( 'show_featured_image', __('Show Unit & Lesson Featured Image?', ZippyCoursesTheme::TEXTDOMAIN), '' )->set_choices( array( '1' => __('Yes', ZippyCoursesTheme::TEXTDOMAIN), '0' => __('No', ZippyCoursesTheme::TEXTDOMAIN) ) )->set_type( 'select' );

        $directory  = $zippycourses->add_section('course_directory', __('Course Directory', ZippyCoursesTheme::TEXTDOMAIN));
        $directory->add_control( 'show_excerpt',__('Show Unit & Lesson Excerpt?', ZippyCoursesTheme::TEXTDOMAIN), '' )->set_choices( array( '1' => __('Yes', ZippyCoursesTheme::TEXTDOMAIN), '0' => __('No', ZippyCoursesTheme::TEXTDOMAIN)))->set_type( 'select' );
        $directory->add_control( 'show_featured_image', __('Show Unit & Lesson Featured Image?', ZippyCoursesTheme::TEXTDOMAIN), '' )->set_choices( array( '1' => __('Yes', ZippyCoursesTheme::TEXTDOMAIN), '0' => __('No', ZippyCoursesTheme::TEXTDOMAIN) ) )->set_type( 'select' );
    */
    $zc->register();

}
add_action( 'customize_register', 'zippy_theme_customizer_setup' );

function zippy_courses_theme_metaboxes($metaboxes)
{
    $dir    = dirname(__FILE__) . '/assets/views/metaboxes/';
    $files  = array_diff(scandir($dir), array('..', '.'));

    foreach ($files as $key => &$file) {
        if (!is_dir($dir . $file)) {
            $file = $dir . $file;
        } else {
            unset($files[$key]);
        }
    }

    return array_merge($metaboxes, $files);
}
add_filter('zippy_metaboxes', 'zippy_courses_theme_metaboxes');

function loginForm($fields, $form_id)
{
    if ($form_id == 'login') {
        foreach ($fields->all() as $field) {
            $field->setLabelVisibility(false);
        }
    }

    return $fields;
}
add_filter('zippy_form_fields', 'loginForm', 10, 2);

add_action('zippy_filter_include_customizer_styles', '__return_false');

